@@ -24,12 +24,6 @@
 #include "meta-draw-op-private.h"
 #include "meta-theme-impl-private.h"
 
-#define GDK_COLOR_RGBA(color)                            \
-        ((guint32) (0xff                               | \
-                    ((int)((color).red * 255) << 24)   | \
-                    ((int)((color).green * 255) << 16) | \
-                    ((int)((color).blue * 255) << 8)))
-
 #define GDK_COLOR_RGB(color)                             \
         ((guint32) (((int)((color).red * 255) << 16)   | \
                     ((int)((color).green * 255) << 8)  | \
@@ -57,11 +51,11 @@ struct _MetaDrawOpList
 static void
 fill_env (MetaPositionExprEnv *env,
           const MetaDrawInfo  *info,
-          GdkRectangle         logical_region)
+          MetaRectangleDouble  rect)
 {
   /* FIXME this stuff could be raised into draw_op_list_draw() probably
    */
-  env->rect = logical_region;
+  env->rect = rect;
   env->object_width = -1;
   env->object_height = -1;
 
@@ -69,8 +63,8 @@ fill_env (MetaPositionExprEnv *env,
   env->right_width = info->right_width;
   env->top_height = info->top_height;
   env->bottom_height = info->bottom_height;
-  env->frame_x_center = info->width / 2 - logical_region.x;
-  env->frame_y_center = info->height / 2 - logical_region.y;
+  env->frame_x_center = info->width / 2 - rect.x;
+  env->frame_y_center = info->height / 2 - rect.y;
 
   env->mini_icon_width = info->mini_icon ? gdk_pixbuf_get_width (info->mini_icon) : 0;
   env->mini_icon_height = info->mini_icon ? gdk_pixbuf_get_height (info->mini_icon) : 0;
@@ -81,223 +75,63 @@ fill_env (MetaPositionExprEnv *env,
   env->title_height = info->title_layout_height;
 }
 
-static GdkPixbuf*
-pixbuf_tile (GdkPixbuf *tile,
-             int        width,
-             int        height)
+static cairo_surface_t *
+scale_surface (GdkPixbuf         *src,
+               MetaImageFillType  fill_type,
+               gdouble            width,
+               gdouble            height,
+               gboolean           vertical_stripes,
+               gboolean           horizontal_stripes)
 {
-  GdkPixbuf *pixbuf;
-  int tile_width;
-  int tile_height;
-  int i, j;
+  gdouble pixbuf_width;
+  gdouble pixbuf_height;
+  cairo_surface_t *src_surface;
+  cairo_surface_t *new_surface;
+  cairo_t *cr;
 
-  tile_width = gdk_pixbuf_get_width (tile);
-  tile_height = gdk_pixbuf_get_height (tile);
+  pixbuf_width = gdk_pixbuf_get_width (src);
+  pixbuf_height = gdk_pixbuf_get_height (src);
 
-  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-                           gdk_pixbuf_get_has_alpha (tile),
-                           8, width, height);
+  src_surface = gdk_cairo_surface_create_from_pixbuf (src, 1, NULL);
+  new_surface = cairo_surface_create_similar (src_surface,
+                                              CAIRO_CONTENT_COLOR_ALPHA,
+                                              ceil (width), ceil (height));
 
-  i = 0;
-  while (i < width)
+  cr = cairo_create (new_surface);
+
+  if ((pixbuf_width == width && pixbuf_height == height) ||
+      fill_type == META_IMAGE_FILL_TILE)
     {
-      j = 0;
-      while (j < height)
-        {
-          int w, h;
+      cairo_set_source_surface (cr, src_surface, 0, 0);
 
-          w = MIN (tile_width, width - i);
-          h = MIN (tile_height, height - j);
-
-          gdk_pixbuf_copy_area (tile,
-                                0, 0,
-                                w, h,
-                                pixbuf,
-                                i, j);
-
-          j += tile_height;
-        }
-
-      i += tile_width;
-    }
-
-  return pixbuf;
-}
-
-static GdkPixbuf *
-replicate_rows (GdkPixbuf  *src,
-                int         src_x,
-                int         src_y,
-                int         width,
-                int         height)
-{
-  unsigned int n_channels = gdk_pixbuf_get_n_channels (src);
-  unsigned int src_rowstride = gdk_pixbuf_get_rowstride (src);
-  unsigned char *pixels = (gdk_pixbuf_get_pixels (src) + src_y * src_rowstride + src_x
-                           * n_channels);
-  unsigned char *dest_pixels;
-  GdkPixbuf *result;
-  unsigned int dest_rowstride;
-  int i;
-
-  result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, n_channels == 4, 8,
-                           width, height);
-  dest_rowstride = gdk_pixbuf_get_rowstride (result);
-  dest_pixels = gdk_pixbuf_get_pixels (result);
-
-  for (i = 0; i < height; i++)
-    memcpy (dest_pixels + dest_rowstride * i, pixels, n_channels * width);
-
-  return result;
-}
-
-static GdkPixbuf *
-replicate_cols (GdkPixbuf  *src,
-                int         src_x,
-                int         src_y,
-                int         width,
-                int         height)
-{
-  unsigned int n_channels = gdk_pixbuf_get_n_channels (src);
-  unsigned int src_rowstride = gdk_pixbuf_get_rowstride (src);
-  unsigned char *pixels = (gdk_pixbuf_get_pixels (src) + src_y * src_rowstride + src_x
-                           * n_channels);
-  unsigned char *dest_pixels;
-  GdkPixbuf *result;
-  unsigned int dest_rowstride;
-  int i, j;
-
-  result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, n_channels == 4, 8,
-                           width, height);
-  dest_rowstride = gdk_pixbuf_get_rowstride (result);
-  dest_pixels = gdk_pixbuf_get_pixels (result);
-
-  for (i = 0; i < height; i++)
-    {
-      unsigned char *p = dest_pixels + dest_rowstride * i;
-      unsigned char *q = pixels + src_rowstride * i;
-
-      unsigned char r = *(q++);
-      unsigned char g = *(q++);
-      unsigned char b = *(q++);
-
-      if (n_channels == 4)
-        {
-          unsigned char a;
-
-          a = *(q++);
-
-          for (j = 0; j < width; j++)
-            {
-              *(p++) = r;
-              *(p++) = g;
-              *(p++) = b;
-              *(p++) = a;
-            }
-        }
-      else
-        {
-          for (j = 0; j < width; j++)
-            {
-              *(p++) = r;
-              *(p++) = g;
-              *(p++) = b;
-            }
-        }
-    }
-
-  return result;
-}
-
-static GdkPixbuf*
-scale_and_alpha_pixbuf (GdkPixbuf             *src,
-                        MetaAlphaGradientSpec *alpha_spec,
-                        MetaImageFillType      fill_type,
-                        int                    width,
-                        int                    height,
-                        gboolean               vertical_stripes,
-                        gboolean               horizontal_stripes)
-{
-  GdkPixbuf *pixbuf;
-  GdkPixbuf *temp_pixbuf;
-
-  pixbuf = NULL;
-
-  pixbuf = src;
-
-  if (gdk_pixbuf_get_width (pixbuf) == width &&
-      gdk_pixbuf_get_height (pixbuf) == height)
-    {
-      g_object_ref (G_OBJECT (pixbuf));
+      if (fill_type == META_IMAGE_FILL_TILE)
+        cairo_pattern_set_extend (cairo_get_source (cr), CAIRO_EXTEND_REPEAT);
     }
   else
     {
-      if (fill_type == META_IMAGE_FILL_TILE)
-        {
-          pixbuf = pixbuf_tile (pixbuf, width, height);
-        }
-      else
-        {
-          int src_h, src_w, dest_h, dest_w;
-          src_h = gdk_pixbuf_get_height (src);
-          src_w = gdk_pixbuf_get_width (src);
+      gdouble scale_x;
+      gdouble scale_y;
 
-          /* prefer to replicate_cols if possible, as that
-           * is faster (no memory reads)
-           */
-          if (horizontal_stripes)
-            {
-              dest_w = gdk_pixbuf_get_width (src);
-              dest_h = height;
-            }
-          else if (vertical_stripes)
-            {
-              dest_w = width;
-              dest_h = gdk_pixbuf_get_height (src);
-            }
+      scale_x = width / pixbuf_width;
+      scale_y = height / pixbuf_height;
 
-          else
-            {
-              dest_w = width;
-              dest_h = height;
-            }
+      if (vertical_stripes)
+        scale_y = 1;
+      else if (horizontal_stripes)
+        scale_x = 1;
 
-          if (dest_w == src_w && dest_h == src_h)
-            {
-              temp_pixbuf = src;
-              g_object_ref (G_OBJECT (temp_pixbuf));
-            }
-          else
-            {
-              temp_pixbuf = gdk_pixbuf_scale_simple (src,
-                                                     dest_w, dest_h,
-                                                     GDK_INTERP_BILINEAR);
-            }
+      cairo_scale (cr, scale_x, scale_y);
+      cairo_set_source_surface (cr, src_surface, 0, 0);
 
-          /* prefer to replicate_cols if possible, as that
-           * is faster (no memory reads)
-           */
-          if (horizontal_stripes)
-            {
-              pixbuf = replicate_cols (temp_pixbuf, 0, 0, width, height);
-              g_object_unref (G_OBJECT (temp_pixbuf));
-            }
-          else if (vertical_stripes)
-            {
-              pixbuf = replicate_rows (temp_pixbuf, 0, 0, width, height);
-              g_object_unref (G_OBJECT (temp_pixbuf));
-            }
-          else
-            {
-              pixbuf = temp_pixbuf;
-            }
-        }
+      if (vertical_stripes || horizontal_stripes)
+        cairo_pattern_set_extend (cairo_get_source (cr), CAIRO_EXTEND_REPEAT);
     }
 
-  if (pixbuf)
-    pixbuf = meta_alpha_gradient_spec_apply_alpha (alpha_spec, pixbuf, pixbuf == src);
+  cairo_paint (cr);
+  cairo_destroy (cr);
+  cairo_surface_destroy (src_surface);
 
-  return pixbuf;
+  return new_surface;
 }
 
 static GdkPixbuf *
@@ -378,20 +212,16 @@ colorize_pixbuf (GdkPixbuf *orig,
   return pixbuf;
 }
 
-static GdkPixbuf *
-draw_op_as_pixbuf (const MetaDrawOp   *op,
-                   GtkStyleContext    *context,
-                   const MetaDrawInfo *info,
-                   int                 width,
-                   int                 height)
+static cairo_surface_t *
+draw_op_as_surface (const MetaDrawOp   *op,
+                    GtkStyleContext    *context,
+                    const MetaDrawInfo *info,
+                    gdouble             width,
+                    gdouble             height)
 {
-  /* Try to get the op as a pixbuf, assuming w/h in the op
-   * matches the width/height passed in. return NULL
-   * if the op can't be converted to an equivalent pixbuf.
-   */
-  GdkPixbuf *pixbuf;
+  cairo_surface_t *surface;
 
-  pixbuf = NULL;
+  surface = NULL;
 
   switch (op->type)
     {
@@ -399,20 +229,6 @@ draw_op_as_pixbuf (const MetaDrawOp   *op,
       break;
 
     case META_DRAW_RECTANGLE:
-      if (op->data.rectangle.filled)
-        {
-          GdkRGBA color;
-
-          meta_color_spec_render (op->data.rectangle.color_spec,
-                                  context,
-                                  &color);
-
-          pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-                                   FALSE,
-                                   8, width, height);
-
-          gdk_pixbuf_fill (pixbuf, GDK_COLOR_RGBA (color));
-        }
       break;
 
     case META_DRAW_ARC:
@@ -422,32 +238,16 @@ draw_op_as_pixbuf (const MetaDrawOp   *op,
       break;
 
     case META_DRAW_TINT:
-      {
-        GdkRGBA color;
-
-        meta_color_spec_render (op->data.rectangle.color_spec,
-                                context, &color);
-
-        pixbuf = meta_alpha_gradient_spec_render (op->data.tint.alpha_spec,
-                                                  width, height, color);
-      }
       break;
 
     case META_DRAW_GRADIENT:
-      {
-        pixbuf = meta_gradient_spec_render (op->data.gradient.gradient_spec,
-                                            context, width, height);
-
-        pixbuf = meta_alpha_gradient_spec_apply_alpha (op->data.gradient.alpha_spec,
-                                                       pixbuf, FALSE);
-      }
       break;
 
     case META_DRAW_IMAGE:
       {
-	if (op->data.image.colorize_spec)
-	  {
-	    GdkRGBA color;
+        if (op->data.image.colorize_spec)
+          {
+            GdkRGBA color;
 
             meta_color_spec_render (op->data.image.colorize_spec,
                                     context, &color);
@@ -468,23 +268,21 @@ draw_op_as_pixbuf (const MetaDrawOp   *op,
 
             if (op->data.image.colorize_cache_pixbuf)
               {
-                pixbuf = scale_and_alpha_pixbuf (op->data.image.colorize_cache_pixbuf,
-                                                 op->data.image.alpha_spec,
-                                                 op->data.image.fill_type,
-                                                 width, height,
-                                                 op->data.image.vertical_stripes,
-                                                 op->data.image.horizontal_stripes);
+                surface = scale_surface (op->data.image.colorize_cache_pixbuf,
+                                         op->data.image.fill_type,
+                                         width, height,
+                                         op->data.image.vertical_stripes,
+                                         op->data.image.horizontal_stripes);
               }
-	  }
-	else
-	  {
-	    pixbuf = scale_and_alpha_pixbuf (op->data.image.pixbuf,
-                                             op->data.image.alpha_spec,
-                                             op->data.image.fill_type,
-                                             width, height,
-                                             op->data.image.vertical_stripes,
-                                             op->data.image.horizontal_stripes);
-	  }
+          }
+        else
+          {
+            surface = scale_surface (op->data.image.pixbuf,
+                                     op->data.image.fill_type,
+                                     width, height,
+                                     op->data.image.vertical_stripes,
+                                     op->data.image.horizontal_stripes);
+          }
         break;
       }
 
@@ -497,17 +295,11 @@ draw_op_as_pixbuf (const MetaDrawOp   *op,
       if (info->mini_icon &&
           width <= gdk_pixbuf_get_width (info->mini_icon) &&
           height <= gdk_pixbuf_get_height (info->mini_icon))
-        pixbuf = scale_and_alpha_pixbuf (info->mini_icon,
-                                         op->data.icon.alpha_spec,
-                                         op->data.icon.fill_type,
-                                         width, height,
-                                         FALSE, FALSE);
+        surface = scale_surface (info->mini_icon, op->data.icon.fill_type,
+                                 width, height, FALSE, FALSE);
       else if (info->icon)
-        pixbuf = scale_and_alpha_pixbuf (info->icon,
-                                         op->data.icon.alpha_spec,
-                                         op->data.icon.fill_type,
-                                         width, height,
-                                         FALSE, FALSE);
+        surface = scale_surface (info->icon, op->data.icon.fill_type,
+                                 width, height, FALSE, FALSE);
       break;
 
     case META_DRAW_TITLE:
@@ -523,7 +315,7 @@ draw_op_as_pixbuf (const MetaDrawOp   *op,
       break;
     }
 
-  return pixbuf;
+  return surface;
 }
 
 /* This code was originally rendering anti-aliased using X primitives, and
@@ -542,7 +334,6 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
                        GtkStyleContext     *context,
                        cairo_t             *cr,
                        const MetaDrawInfo  *info,
-                       GdkRectangle         rect,
                        MetaPositionExprEnv *env)
 {
   GdkRGBA color;
@@ -555,7 +346,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
     {
     case META_DRAW_LINE:
       {
-        int x1, x2, y1, y2;
+        gdouble x1, x2, y1, y2;
 
         meta_color_spec_render (op->data.line.color_spec, context, &color);
         gdk_cairo_set_source_rgba (cr, &color);
@@ -575,9 +366,8 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
         x1 = meta_draw_spec_parse_x_position (op->data.line.x1, env);
         y1 = meta_draw_spec_parse_y_position (op->data.line.y1, env);
 
-        if (!op->data.line.x2 &&
-            !op->data.line.y2 &&
-            op->data.line.width==0)
+        if (!op->data.line.x2 && !op->data.line.y2 &&
+            op->data.line.width == 0)
           {
             cairo_rectangle (cr, x1, y1, 1, 1);
             cairo_fill (cr);
@@ -630,7 +420,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_RECTANGLE:
       {
-        int rx, ry, rwidth, rheight;
+        gdouble rx, ry, rwidth, rheight;
 
         meta_color_spec_render (op->data.rectangle.color_spec, context, &color);
         gdk_cairo_set_source_rgba (cr, &color);
@@ -658,7 +448,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_ARC:
       {
-        int rx, ry, rwidth, rheight;
+        gdouble rx, ry, rwidth, rheight;
         double start_angle, end_angle;
         double center_x, center_y;
 
@@ -703,67 +493,42 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_TINT:
       {
-        int rx, ry, rwidth, rheight;
-        gboolean needs_alpha;
-
-        needs_alpha = meta_alpha_gradient_spec_needs_alpha (op->data.tint.alpha_spec);
+        gdouble rx, ry, rwidth, rheight;
 
         rx = meta_draw_spec_parse_x_position (op->data.tint.x, env);
         ry = meta_draw_spec_parse_y_position (op->data.tint.y, env);
         rwidth = meta_draw_spec_parse_size (op->data.tint.width, env);
         rheight = meta_draw_spec_parse_size (op->data.tint.height, env);
 
-        if (!needs_alpha)
-          {
-            meta_color_spec_render (op->data.tint.color_spec, context, &color);
-            gdk_cairo_set_source_rgba (cr, &color);
-
-            cairo_rectangle (cr, rx, ry, rwidth, rheight);
-            cairo_fill (cr);
-          }
-        else
-          {
-            GdkPixbuf *pixbuf;
-
-            pixbuf = draw_op_as_pixbuf (op, context, info, rwidth, rheight);
-
-            if (pixbuf)
-              {
-                gdk_cairo_set_source_pixbuf (cr, pixbuf, rx, ry);
-                cairo_paint (cr);
-
-                g_object_unref (G_OBJECT (pixbuf));
-              }
-          }
+        meta_color_spec_render (op->data.tint.color_spec, context, &color);
+        meta_alpha_gradient_spec_render (op->data.tint.alpha_spec, color, cr,
+                                         rx, ry, rwidth, rheight);
       }
       break;
 
     case META_DRAW_GRADIENT:
       {
-        int rx, ry, rwidth, rheight;
-        GdkPixbuf *pixbuf;
+        gdouble rx, ry, rwidth, rheight;
 
         rx = meta_draw_spec_parse_x_position (op->data.gradient.x, env);
         ry = meta_draw_spec_parse_y_position (op->data.gradient.y, env);
         rwidth = meta_draw_spec_parse_size (op->data.gradient.width, env);
         rheight = meta_draw_spec_parse_size (op->data.gradient.height, env);
 
-        pixbuf = draw_op_as_pixbuf (op, context, info, rwidth, rheight);
-
-        if (pixbuf)
-          {
-            gdk_cairo_set_source_pixbuf (cr, pixbuf, rx, ry);
-            cairo_paint (cr);
-
-            g_object_unref (G_OBJECT (pixbuf));
-          }
+        meta_gradient_spec_render (op->data.gradient.gradient_spec,
+                                   op->data.gradient.alpha_spec,
+                                   cr, context, rx, ry, rwidth, rheight);
       }
       break;
 
     case META_DRAW_IMAGE:
       {
-        int rx, ry, rwidth, rheight;
-        GdkPixbuf *pixbuf;
+        gdouble scale;
+        gdouble rx, ry, rwidth, rheight;
+        cairo_surface_t *surface;
+
+        scale = get_window_scaling_factor ();
+        cairo_scale (cr, 1.0 / scale, 1.0 / scale);
 
         if (op->data.image.pixbuf)
           {
@@ -771,27 +536,43 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
             env->object_height = gdk_pixbuf_get_height (op->data.image.pixbuf);
           }
 
-        rwidth = meta_draw_spec_parse_size (op->data.image.width, env);
-        rheight = meta_draw_spec_parse_size (op->data.image.height, env);
+        rwidth = meta_draw_spec_parse_size (op->data.image.width, env) * scale;
+        rheight = meta_draw_spec_parse_size (op->data.image.height, env) * scale;
 
-        pixbuf = draw_op_as_pixbuf (op, context, info, rwidth, rheight);
+        surface = draw_op_as_surface (op, context, info, rwidth, rheight);
 
-        if (pixbuf)
+        if (surface)
           {
-            rx = meta_draw_spec_parse_x_position (op->data.image.x, env);
-            ry = meta_draw_spec_parse_y_position (op->data.image.y, env);
+            rx = meta_draw_spec_parse_x_position (op->data.image.x, env) * scale;
+            ry = meta_draw_spec_parse_y_position (op->data.image.y, env) * scale;
 
-            gdk_cairo_set_source_pixbuf (cr, pixbuf, rx, ry);
-            cairo_paint (cr);
+            cairo_set_source_surface (cr, surface, rx, ry);
 
-            g_object_unref (G_OBJECT (pixbuf));
+            if (op->data.image.alpha_spec)
+              {
+                cairo_pattern_t *pattern;
+
+                cairo_translate (cr, rx, ry);
+                cairo_scale (cr, rwidth, rheight);
+
+                pattern = meta_alpha_gradient_spec_get_mask (op->data.image.alpha_spec);
+                cairo_mask (cr, pattern);
+
+                cairo_pattern_destroy (pattern);
+              }
+            else
+              {
+                cairo_paint (cr);
+              }
+
+            cairo_surface_destroy (surface);
           }
       }
       break;
 
     case META_DRAW_GTK_ARROW:
       {
-        int rx, ry, rwidth, rheight;
+        gdouble rx, ry, rwidth, rheight;
         double angle = 0, size;
 
         rx = meta_draw_spec_parse_x_position (op->data.gtk_arrow.x, env);
@@ -828,7 +609,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_GTK_BOX:
       {
-        int rx, ry, rwidth, rheight;
+        gdouble rx, ry, rwidth, rheight;
 
         rx = meta_draw_spec_parse_x_position (op->data.gtk_box.x, env);
         ry = meta_draw_spec_parse_y_position (op->data.gtk_box.y, env);
@@ -843,7 +624,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_GTK_VLINE:
       {
-        int rx, ry1, ry2;
+        gdouble rx, ry1, ry2;
 
         rx = meta_draw_spec_parse_x_position (op->data.gtk_vline.x, env);
         ry1 = meta_draw_spec_parse_y_position (op->data.gtk_vline.y1, env);
@@ -856,23 +637,39 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_ICON:
       {
-        int rx, ry, rwidth, rheight;
-        GdkPixbuf *pixbuf;
+        gdouble rx, ry, rwidth, rheight;
+        cairo_surface_t *surface;
 
         rwidth = meta_draw_spec_parse_size (op->data.icon.width, env);
         rheight = meta_draw_spec_parse_size (op->data.icon.height, env);
 
-        pixbuf = draw_op_as_pixbuf (op, context, info, rwidth, rheight);
+        surface = draw_op_as_surface (op, context, info, rwidth, rheight);
 
-        if (pixbuf)
+        if (surface)
           {
             rx = meta_draw_spec_parse_x_position (op->data.icon.x, env);
             ry = meta_draw_spec_parse_y_position (op->data.icon.y, env);
 
-            gdk_cairo_set_source_pixbuf (cr, pixbuf, rx, ry);
-            cairo_paint (cr);
+            cairo_set_source_surface (cr, surface, rx, ry);
 
-            g_object_unref (G_OBJECT (pixbuf));
+            if (op->data.icon.alpha_spec)
+              {
+                cairo_pattern_t *pattern;
+
+                cairo_translate (cr, rx, ry);
+                cairo_scale (cr, rwidth, rheight);
+
+                pattern = meta_alpha_gradient_spec_get_mask (op->data.icon.alpha_spec);
+                cairo_mask (cr, pattern);
+
+                cairo_pattern_destroy (pattern);
+              }
+            else
+              {
+                cairo_paint (cr);
+              }
+
+            cairo_surface_destroy (surface);
           }
       }
       break;
@@ -880,7 +677,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
     case META_DRAW_TITLE:
       if (info->title_layout)
         {
-          int rx, ry;
+          gdouble rx, ry;
           PangoRectangle ink_rect, logical_rect;
 
           meta_color_spec_render (op->data.title.color_spec, context, &color);
@@ -891,7 +688,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
           if (op->data.title.ellipsize_width)
             {
-              int ellipsize_width;
+              gdouble ellipsize_width;
               int right_bearing;
 
               ellipsize_width = meta_draw_spec_parse_x_position (op->data.title.ellipsize_width, env);
@@ -959,7 +756,7 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_OP_LIST:
       {
-        GdkRectangle d_rect;
+        MetaRectangleDouble d_rect;
 
         d_rect.x = meta_draw_spec_parse_x_position (op->data.op_list.x, env);
         d_rect.y = meta_draw_spec_parse_y_position (op->data.op_list.y, env);
@@ -973,9 +770,9 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
 
     case META_DRAW_TILE:
       {
-        int rx, ry, rwidth, rheight;
-        int tile_xoffset, tile_yoffset;
-        GdkRectangle tile;
+        gdouble rx, ry, rwidth, rheight;
+        gdouble tile_xoffset, tile_yoffset;
+        MetaRectangleDouble tile;
 
         rx = meta_draw_spec_parse_x_position (op->data.tile.x, env);
         ry = meta_draw_spec_parse_y_position (op->data.tile.y, env);
@@ -990,8 +787,8 @@ draw_op_draw_with_env (const MetaDrawOp    *op,
         tile_xoffset = meta_draw_spec_parse_x_position (op->data.tile.tile_xoffset, env);
         tile_yoffset = meta_draw_spec_parse_y_position (op->data.tile.tile_yoffset, env);
         /* tile offset should not include x/y */
-        tile_xoffset -= rect.x;
-        tile_yoffset -= rect.y;
+        tile_xoffset -= env->rect.x;
+        tile_yoffset -= env->rect.y;
 
         tile.width = meta_draw_spec_parse_size (op->data.tile.tile_width, env);
         tile.height = meta_draw_spec_parse_size (op->data.tile.tile_height, env);
@@ -1309,7 +1106,7 @@ meta_draw_op_list_draw_with_style (const MetaDrawOpList *op_list,
                                    GtkStyleContext      *context,
                                    cairo_t              *cr,
                                    const MetaDrawInfo   *info,
-                                   GdkRectangle          rect)
+                                   MetaRectangleDouble   rect)
 {
   int i;
   MetaPositionExprEnv env;
@@ -1348,7 +1145,7 @@ meta_draw_op_list_draw_with_style (const MetaDrawOpList *op_list,
         }
       else if (gdk_cairo_get_clip_rectangle (cr, NULL))
         {
-          draw_op_draw_with_env (op, context, cr, info, rect, &env);
+          draw_op_draw_with_env (op, context, cr, info, &env);
         }
     }
 
