@@ -33,28 +33,6 @@
 
 typedef struct _MetaResizePopup MetaResizePopup;
 
-typedef enum
-{
-  META_FRAME_ALLOWS_DELETE            = 1 << 0,
-  META_FRAME_ALLOWS_MENU              = 1 << 1,
-  META_FRAME_ALLOWS_APPMENU           = 1 << 2,
-  META_FRAME_ALLOWS_MINIMIZE          = 1 << 3,
-  META_FRAME_ALLOWS_MAXIMIZE          = 1 << 4,
-  META_FRAME_ALLOWS_VERTICAL_RESIZE   = 1 << 5,
-  META_FRAME_ALLOWS_HORIZONTAL_RESIZE = 1 << 6,
-  META_FRAME_HAS_FOCUS                = 1 << 7,
-  META_FRAME_SHADED                   = 1 << 8,
-  META_FRAME_STUCK                    = 1 << 9,
-  META_FRAME_MAXIMIZED                = 1 << 10,
-  META_FRAME_ALLOWS_SHADE             = 1 << 11,
-  META_FRAME_ALLOWS_MOVE              = 1 << 12,
-  META_FRAME_FULLSCREEN               = 1 << 13,
-  META_FRAME_IS_FLASHING              = 1 << 14,
-  META_FRAME_ABOVE                    = 1 << 15,
-  META_FRAME_TILED_LEFT               = 1 << 16,
-  META_FRAME_TILED_RIGHT              = 1 << 17
-} MetaFrameFlags;
-
 typedef enum
 {
   META_MENU_OP_NONE        = 0,
@@ -164,18 +142,6 @@ typedef enum
 
 } MetaCursor;
 
-typedef enum
-{
-  META_FRAME_TYPE_NORMAL,
-  META_FRAME_TYPE_DIALOG,
-  META_FRAME_TYPE_MODAL_DIALOG,
-  META_FRAME_TYPE_UTILITY,
-  META_FRAME_TYPE_MENU,
-  META_FRAME_TYPE_BORDER,
-  META_FRAME_TYPE_ATTACHED,
-  META_FRAME_TYPE_LAST
-} MetaFrameType;
-
 typedef enum
 {
   /* Create gratuitous divergence from regular
@@ -228,57 +194,6 @@ typedef enum
   META_SIDE_BOTTOM          = META_DIRECTION_BOTTOM
 } MetaSide;
 
-/* Function a window button can have.  Note, you can't add stuff here
- * without extending the theme format to draw a new function and
- * breaking all existing themes.
- */
-typedef enum
-{
-  META_BUTTON_FUNCTION_MENU,
-  META_BUTTON_FUNCTION_APPMENU,
-  META_BUTTON_FUNCTION_MINIMIZE,
-  META_BUTTON_FUNCTION_MAXIMIZE,
-  META_BUTTON_FUNCTION_CLOSE,
-  META_BUTTON_FUNCTION_SHADE,
-  META_BUTTON_FUNCTION_ABOVE,
-  META_BUTTON_FUNCTION_STICK,
-  META_BUTTON_FUNCTION_UNSHADE,
-  META_BUTTON_FUNCTION_UNABOVE,
-  META_BUTTON_FUNCTION_UNSTICK,
-  META_BUTTON_FUNCTION_LAST
-} MetaButtonFunction;
-
-#define MAX_BUTTONS_PER_CORNER META_BUTTON_FUNCTION_LAST
-
-typedef struct _MetaButtonLayout MetaButtonLayout;
-struct _MetaButtonLayout
-{
-  /* buttons in the group on the left side */
-  MetaButtonFunction left_buttons[MAX_BUTTONS_PER_CORNER];
-  gboolean left_buttons_has_spacer[MAX_BUTTONS_PER_CORNER];
-
-  /* buttons in the group on the right side */
-  MetaButtonFunction right_buttons[MAX_BUTTONS_PER_CORNER];
-  gboolean right_buttons_has_spacer[MAX_BUTTONS_PER_CORNER];
-};
-
-typedef struct _MetaFrameBorders MetaFrameBorders;
-struct _MetaFrameBorders
-{
-  /* The frame border is made up of two pieces - an inner visible portion
-   * and an outer portion that is invisible but responds to events.
-   */
-  GtkBorder visible;
-  GtkBorder invisible;
-
-  /* For convenience, we have a "total" border which is equal to the sum
-   * of the two borders above. */
-  GtkBorder total;
-};
-
-/* sets all dimensions to zero */
-void meta_frame_borders_clear (MetaFrameBorders *self);
-
 /* should investigate changing these to whatever most apps use */
 #define META_ICON_WIDTH 96
 #define META_ICON_HEIGHT 96
