@@ -49,8 +49,6 @@ void meta_ui_init (int *argc, char ***argv);
 
 Display* meta_ui_get_display (void);
 
-gint     meta_ui_get_screen_number (void);
-
 void meta_ui_add_event_func    (Display       *xdisplay,
                                 MetaEventFunc  func,
                                 gpointer       data);
@@ -76,14 +74,14 @@ void meta_ui_get_frame_mask (MetaUI  *ui,
                              guint    height,
                              cairo_t *cr);
 
-Window meta_ui_create_frame_window (MetaUI  *ui,
+Window meta_ui_create_frame_window (MetaUI *ui,
                                     Display *xdisplay,
-                                    Visual  *xvisual,
-                                    gint     x,
-                                    gint     y,
-                                    gint     width,
-                                    gint     height);
-
+                                    Visual *xvisual,
+				    gint x,
+				    gint y,
+				    gint width,
+				    gint height,
+				    gint screen_no);
 void meta_ui_destroy_frame_window (MetaUI *ui,
 				   Window  xwindow);
 void meta_ui_move_resize_frame (MetaUI *ui,
