@@ -18,8 +18,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
-#include "gradient.h"
 #include <gtk/gtk.h>
+#include <libmetacity/meta-gradient-private.h>
 
 typedef void (* RenderGradientFunc) (cairo_t     *cr,
                                      int          width,
